/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak.realm;

import java.util.Objects;
import java.util.Optional;

import org.keycloak.admin.client.Keycloak;
import org.keycloak.representations.idm.RealmRepresentation;
import org.keycloak.representations.idm.RoleRepresentation;
import org.keycloak.representations.userprofile.config.UPConfig;
import org.springframework.stereotype.Component;

import de.ozgcloud.operator.keycloak.KeycloakException;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Component
class KeycloakRealmRemoteService {

	private final Keycloak keycloak;

	public void createRealm(RealmRepresentation realm) {
		keycloak.realms().create(realm);
	}

	public void deleteRealm(String realmName) {
		keycloak.realm(realmName).remove();
	}

	public void updateRealm(RealmRepresentation realm) {
		keycloak.realm(realm.getRealm()).update(realm);
	}

	public Optional<RoleRepresentation> getRealmRole(String roleName, String realmName) {
		return Optional.ofNullable(keycloak.realm(realmName))
				.orElseThrow(() -> new KeycloakException("Realm with Name " + realmName + " not found."))
				.roles()
				.list()
				.stream().filter(role -> Objects.equals(roleName, role.getName()))
				.findFirst();
	}

	public void updateRealmRole(RoleRepresentation role, String realm) {
		keycloak.realm(realm).roles().get(role.getName()).update(role);
	}

	public void addRealmRole(RoleRepresentation role, String realm) {
		keycloak.realm(realm).roles().create(role);
	}

	UPConfig getUserProfileConfig(RealmRepresentation realmRepresentation) {
		return keycloak.realm(realmRepresentation.getRealm())
				.users()
				.userProfile()
				.getConfiguration();
	}

	void updateUserProfileConfig(RealmRepresentation realmRepresentation, UPConfig userProfileConfig) {
		keycloak.realm(realmRepresentation.getRealm())
				.users()
				.userProfile()
				.update(userProfileConfig);
	}
}
