package de.ozgcloud.operator.keycloak.user;

import java.util.Optional;

import org.springframework.stereotype.Component;

import io.fabric8.kubernetes.api.model.Secret;
import io.fabric8.kubernetes.client.dsl.Resource;
import io.fabric8.kubernetes.client.extension.ResourceAdapter;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Component
class UserSecretService {

	private final UserNameConverter userNameConverter;
	private final UserSecretBuilder secretBuilder;
	private final UserSecretReader secretReader;
	private final KubernetesRemoteService kubernetesRemoteService;

	public Secret create(OzgCloudKeycloakUserSpec userSpec, String namespace) {
		var secretName = userNameConverter.toSecretName(userSpec.getKeycloakUser());

		var credentialsSecret = secretBuilder.build(secretName, userSpec.getKeycloakUser(), namespace);

		var adapter = createResourceAdpater(getUserSecret(userSpec, namespace));
		adapter.create(credentialsSecret);

		return credentialsSecret;
	}

	ResourceAdapter<Secret> createResourceAdpater(Resource<Secret> secretResource) {
		return new ResourceAdapter<>(secretResource);
	}

	Resource<Secret> getUserSecret(OzgCloudKeycloakUserSpec userSpec, String namespace) {
		var secretName = userNameConverter.toSecretName(userSpec.getKeycloakUser());

		return kubernetesRemoteService.getSecret(namespace, secretName);
	}

	public String getPasswordFromSecret(Secret secret) {
		return secretReader.getPasswortFromSecret(secret);
	}

	public Secret getOrCreateClusterSecret(OzgCloudKeycloakUserSpec userSpec, String namespace) {
		return Optional.ofNullable(getUserSecret(userSpec, namespace)).map(Resource::get)
				.orElseGet(() -> create(userSpec, namespace));
	}
}
